@@ -200,6 +200,27 @@ class RollingDeployController:
     def compute_batches(self, rolling_group_docs, nonrolling_group_docs, rolling_divisor):
         return BatchCompute.compute_rolling_batches(rolling_divisor, rolling_group_docs, nonrolling_group_docs)
 
+    def run_hook(self, name, application, build_name, batches, batch_number=None, target=None):
+        args = {
+            "hook_parameters": {
+                "deployment_id": self.deployment_id,
+                "build": build_name
+            }
+        }
+
+        if name == "AUTO_DEPLOYMENT_START":
+            args['hook_parameters']['target'] = target
+            args['hook_parameters']['batches'] = batches
+        if name == "AUTO_DEPLOYMENT_COMPLETE" or name == "AUTO_DEPLOYMENT_FAILED":
+            args['hook_parameters']['deployment'] = self.datasvc.deploysvc.GetDeployment(application,
+                                                                                         self.deployment_id)
+            args['hook_parameters']['batches'] = batches
+        if "AUTO_DEPLOYMENT_BATCH" in name:
+            args['hook_parameters']['batch_number'] = batch_number
+            args['hook_parameters']['batch_count'] = len(batches)
+            args['hook_parameters']['batch'] = batches[batch_number]
+        self.datasvc.actionsvc.hooks.run_hook(application, name, args)
+
     def run(self, application, build_name, target, rolling_divisor, rolling_pause, parallel=True):
         '''
         Run rolling deployment. This should be called iff the deployment is called via groups/environments
@@ -218,6 +239,9 @@ class RollingDeployController:
 
         logging.debug("computed batches: {}".format(batches))
 
+        #run pre hook
+        self.run_hook("AUTO_DEPLOYMENT_START", application, build_name, batches, target=target)
+
         self.datasvc.deploysvc.InitializeDeploymentPlan(application, self.deployment_id, batches, gitrepos)
 
         self.datasvc.jobsvc.NewJobData({
@@ -229,12 +253,20 @@ class RollingDeployController:
 
         for i, b in enumerate(batches):
             logging.debug("doing DeployController.run: deploy_gds: {}".format(b['gitdeploys']))
+
+            #run start hook
+            self.run_hook("AUTO_DEPLOYMENT_BATCH_BEGIN", application, build_name, batches, batch_number=i)
+
             ok, results = self.dc.run(application, build_name, b['servers'], b['gitdeploys'],
                                       parallel=parallel, batch_number=i)
             if not ok:
                 self.datasvc.jobsvc.NewJobData({"RollingDeployment": "error"})
+                self.run_hook("AUTO_DEPLOYMENT_FAILED", application, build_name, batches)
                 return False
 
+            #run batch done hook
+            self.run_hook("AUTO_DEPLOYMENT_BATCH_DONE", application, build_name, batches, batch_number=i)
+
             deploy_doc = self.datasvc.deploysvc.GetDeployment(application, self.deployment_id)
             assert deploy_doc
             if deploy_doc['status'] == 'error':
@@ -247,6 +279,8 @@ class RollingDeployController:
                 logging.debug("RollingDeployController: {}".format(msg))
                 time.sleep(rolling_pause)
 
+        #run post hook
+        self.run_hook("AUTO_DEPLOYMENT_COMPLETE", application, build_name, batches, target=target)
         return True
 
 def determine_deployabe_servers(all_gd_servers, specified_servers):
@@ -350,6 +384,17 @@ def _threadsafe_process_gitdeploy(gddoc, build_doc, settings, job_id, deployment
                 return
             logging.debug("_threadsafe_process_gitdeploy: git commit result: {}".format(str(res)))
 
+            try:
+                commit_hash = gdm.get_latest_commit_hash()
+                datasvc.deploysvc.UpdateDeployment(gddoc['application'], deployment_id,
+                                                   {'commits': {gitrepo_name: str(commit_hash)}})
+                logging.debug("_threadsafe_process_gitdeploy: git commit hash: {}".format(str(commit_hash)))
+            except:
+                exc_msg = str(sys.exc_info()[1]).split('\n')
+                exc_msg.insert(0, "ERROR: get_commit_hash")
+                datasvc.deploysvc.UpdateDeployment_Phase1(gddoc['application'], deployment_id, gitrepo_name,
+                                                          step=exc_msg)
+
             datasvc.jobsvc.NewJobData({"ProcessGitdeploys": {gddoc['name']: "checking diff"}})
             try:
                 res = gdm.inspect_latest_diff()
@@ -637,7 +682,7 @@ class DeployController:
         assert elita.util.type_check.is_seq(gitdeploys)
         assert isinstance(batch_number, int) and batch_number >= 0
 
-        build_doc = self.datasvc.buildsvc.GetBuildDoc(app_name, build_name)
+        build_doc = self.datasvc.buildsvc.GetBuild(app_name, build_name)
         gitdeploy_docs = {gd: self.datasvc.gitsvc.GetGitDeploy(app_name, gd) for gd in gitdeploys}
 
         queue = billiard.Queue()
